function [viterbiPitch,statePath,pObservations,pTransitions] = ...
    CalculateViterbiPitch(fileStruct, pitchModel)
% CALCULATEVITERBIPITCH Smooth the pitch probability data to calculate pitch
% 
% function [viterbiPitch,statePath,pObservations,pTransitions] = ...
%    CalculateViterbiPitch(fileStruct, pitchModel)
% Use the Viterbi algorithm to calculate the pitch of a sound.  This
% routine prepares transition probabilities, calculates p(observation |
% class) and then send it to a Viterbi decoder.  This routine returns the
% best decoded pitch, the array of best paths (one column per frame, one
% row per pitch class, and the final observation and transition
% probability matrices.

global gRandomizeCorrelogramFrames
if exist('gRandomizeCorrelogramFrames','var') && ~isempty(gRandomizeCorrelogramFrames)
    randomizeCorrelogramFrames = gRandomizeCorrelogramFrames;
else
    randomizeCorrelogramFrames = 0;
end

global gShowGraphics
if exist('gShowGraphics','var') && ~isempty(gShowGraphics)
    showGraphics = gShowGraphics;
else
    showGraphics = 1;
end

% Hacks, because we should just always create the right size transition
% matrices (70x70) when we summarize the data....
if size(pitchModel.transitionProbabilities,1) < 70 || size(pitchModel.transitionProbabilities,2) < 70
    pt = zeros(70,70);
    pt(1:size(pitchModel.transitionProbabilities,1), 1:size(pitchModel.transitionProbabilities,2)) = ...
        pitchModel.transitionProbabilities;
    pitchModel.transitionProbabilities = pt;
end
if size(pitchModel.classProbabilities,1) < 70
    pt = zeros(70,1);
    pt(1:size(pitchModel.classProbabilities,1), 1) = pitchModel.classProbabilities;
    pitchModel.classProbabilities = pt;
end
if size(pitchModel.allTransitions,1) < 70 || size(pitchModel.allTransitions,2) < 70
    pt = zeros(70,70);
    pt(1:size(pitchModel.allTransitions,1), 1:size(pitchModel.allTransitions,2)) = ...
        pitchModel.allTransitions;
    pitchModel.allTransitions = pt;
end


% Normalize by class probabilities
% First make sure that we don't divide by zero.. replace all zeros with
% next smallest value.
cp = pitchModel.classProbabilities(:);
goodCp = cp > 0;
cp(~goodCp) = min(cp(goodCp));
quantizedPitchEvidence = fileStruct.mlpCorrelogram ./ ...
    repmat(cp, 1, size(fileStruct.mlpCorrelogram,2));


%%
% Turn the quantized pitch evidence into a real probability distribution
% (so the observation data sums to one at every time frame.)
qpe = quantizedPitchEvidence;
qpe = qpe ./ repmat(sum(qpe,1), size(qpe,1), 1);
tp = pitchModel.transitionProbabilities;
if showGraphics
    subplot(4,1,1);
    imagesc(fileStruct.mlpCorrelogram.^0.5); title('P(o|tau)'); axis xy
    ylabel('Pitch Class');
    subplot(4,1,2);
    imagesc(qpe.^0.5); title('P(tau|o)- Normalized by Priors'); axis xy
    ylabel('Pitch Class');
    subplot(4,1,3);
end

% Test to see if the viterbi transition matrix is helping.  Randomize the
% observation probabilities (random permutation) every 
% N frames to see if transition probabilities can save the day.
if randomizeCorrelogramFrames > 0
    h = size(qpe,1);
    for i=1:randomizeCorrelogramFrames:size(qpe,2)
        qpe(:,i) = qpe(randperm(h),i);
    end
end

%%
% Now do the viterbi decoding.
if length(pitchModel.lambda) == 1
    [bestPath,statePath,pObservations,pTransitions] = ...
            ViterbiDecoder(qpe, tp, pitchModel.lambda);

    viterbiPitch = DequantizePitches(bestPath);
    viterbiPitch(viterbiPitch <= 0) = nan;
else
    viterbiPitch = zeros(length(pitchModel.lambda), size(qpe,2));
    for li = 1:length(pitchModel.lambda)
        l = pitchModel.lambda(li);
        [bestPath,statePath,pObservations,pTransitions] = ViterbiDecoder(qpe, tp, l);

        vp = DequantizePitches(bestPath);
        vp(vp <= 0) = nan;
        viterbiPitch(li,:) = vp';
    end
end

if showGraphics
    vp = viterbiPitch(floor((1+size(viterbiPitch,1))/2), :);
    subplot(4,1,4);
    plot(vp);
    title('Pitch from Correlogram/MLP Model');
    xlabel('Time (centiseconds)'); ylabel('Pitch (Hz)');
    a = axis;
    axis([1 length(vp) a(3) a(4)])
end


function [viterbiPitch,statePath,pObservations,pTransitions] = ...
    TestWithWarbleSound(transitionProbabilities)
load DesignPitchNeuralNet.mat
load correlogramPCA.mat
fs16 = 16000;
t = 0:1/fs16:10;
F0 = 160;
Fd = 80;
Fv = .1;
% http://www.wolframalpha.com/input/?i=integral+of+%28440%2B50*sin%282*pi*4*t%29%29*2*pi&dataset=&equal=Submit
ph = F0*2*pi*t - Fd/Fv*cos(2*pi*Fv*t);
testSignal = sin(ph) + 1/2*sin(2*ph) + 1/3*sin(3*ph) + 1/4*sin(4*ph)+...
    1/5*sin(5*ph) + 1/6*sin(6*ph) + 1/7*sin(7*ph) + 1/8*sin(8*ph);
if 0
    soundsc(testSignal,fs16);
    wavwrite(testSignal/max(testSignal)/1.1, fs16, 'testSignal.wav');
end

testSignalNoise = testSignal + .05*randn(size(t)).*t/max(t);
%%
lambda = 1;
global gShowCorrelogramFrames       % Added by Malcolm 9/9/2012
gShowCorrelogramFrames = 1;

figure(1);
[viterbiPitch,statePath,pObservations,pTransitions] = ...
    CalculateViterbiPitch(testSignalNoise, fs16, transitionProbabilities, lambda, ...
    pcaMeanValues, pcaVectors(:, 1:nPCA), finalGoodNetwork);

%%
figure(2);
plot(1:length(viterbiPitch), viterbiPitch);
legend('Viterbi Pitch');
title('Warble Test Pitch');
xlabel('Time (centiseconds)');
ylabel('Pitch (Hz)');

if 0                % 
%%
    % Some test code..
    groundTruthFile = fileList{1};
    groundTruthData = load(fileList{ 1});
    groundTruthPitch = groundTruthData(:,1);

    soundFile = strrep(strrep(strrep(fileList{1}, '/REF/', '/MIC/'), '/ref_', '/mic_'), 'f0', 'wav');
    [soundData, fs] = audioread(soundFile);
    sound16 = resample(soundData, 160,441); fs16 = 16000;

    lambdaList = 1e-4*(2).^(0:11);
    nPCA = 200;

    [viterbiPitch,statePath,pObservations,pTransitions] = ...
        ViterbiPitchMLP(sound16, fs16, transitionProbabilities, lambdaList, ...
        meanValues, principalComponents(:,1:nPCA), finalGoodNetwork);
%%
end
