function fileList = ComputeAllFeatures(fileListOrName, pitchModel, ...
    indexList, forceCalculation)
% COMPUTEALLFEATURES Compute all the features needed for the pitch calculation
%
% function fileList = ComputeAllFeatures(fileListOrName, pitchModel,indexList) 
%
% Compute all the pitch model features for a single WAVE file or a
% list of fileStructs.  This routine returns a new file struct with the
% computed features in the mlpCorrelogram field.  This can then be used as
% input to the Viterbi search algorithm.
%
% This function needs access to the Auditory Toolbox and Netlab.

global gForceCorrelogramCalculation

if nargin < 4
    if exist('gForceCorrelogramCalculation', 'var') && ~isempty(gForceCorrelogramCalculation)
        forceCalculation = gForceCorrelogramCalculation;
    else
        forceCalculation = 0;
    end
end

if ischar(fileListOrName)
    fileName = fileListOrName;
    correlogramName = [fileName '.mat'];
    fileList = struct(...
        'pitchFile', [], ...
        'waveFile', fileName, ...
        'pitchData', [], ...
        'pitchClass', [], ...
        'pitchRate', 0, ...
        'highEnergy', [], ...
        'correlogram', [], ...
        'correlogramFile', correlogramName, ...
        'correlogramEnergy', [], ...
        'correlogramFrameCount', 0, ...
        'pcaCorrelogram', [], ...
        'mlpCorrelogram', [] ...
        );
    fileList = ComputeAllFeatures(fileList, pitchModel, 1, forceCalculation);
    return;
end

if ~exist('mlpfwd', 'file')
    path(path, '../netlab');
end

fileList = fileListOrName;
if nargin < 3
    indexList = 1:length(fileList);
end

% Nothing to return since correlograms are cached on disk.
global gCorrelogramFrameWidth
gCorrelogramFrameWidth = pitchModel.correlogramFrameWidth;

fileList = ComputeCorrelograms(fileList, indexList, forceCalculation);

for fileNumber=indexList
    if isempty(fileList(fileNumber).mlpCorrelogram)
         if isempty(fileList(fileNumber).correlogram)
            [correlogramData] = RestoreCorrelogram(fileList(fileNumber));
        else
            correlogramData = fileList(fileNumber).correlogram;
        end
        if ~isempty(fileList(fileNumber).pitchData) && ...
                size(correlogramData, 2) ~= size(fileList(fileNumber).pitchData, 2)
            error('ComputeAllFeatures: correlogramData/pitchData size mismatch.');
        end

        pcaPitch = PCATransform(correlogramData, pitchModel.pcaModel);

        quantizedPitchEvidence = mlpfwd(pitchModel.mlpModel, pcaPitch')';
        fileList(fileNumber).mlpCorrelogram = quantizedPitchEvidence;
    end
end
