function fileList = ComputeCorrelograms(fileList, indexList, forceComputation)
% COMPUTECORRELOGRAMS Compute a correlogram for the pitch calculation
%
% function ComputeCorrelograms(fileList, indexList, forceComputation)
%
% Compute the correlograms for all the sound files in the fileList.  The
% indexList variable allows you to specify that only those index values (in
% the fileList) should be computed.  This routine stores the resulting
% correlograms onto disk, near the .WAV file, so we can pull them in
% later.
%
% Only a fixed number of sample rates are accepted by this routine since
% the gammatone filters (and their position) are driven by the
% sound sampling rate.
global gSignalToNoiseRatio

if nargin < 2 || isempty(indexList)
    indexList = 1:length(fileList);
end
if nargin < 3
    forceComputation = 0;
end

if ~exist('MakeERBFilters', 'file')
    path(path,'../AuditoryToolbox');
end

global gCorrelogramFrameWidth
if exist('gCorrelogramFrameWidth', 'var') && ~isempty(gCorrelogramFrameWidth)
    desiredCorrelogramFrameWidth = gCorrelogramFrameWidth;
else
    desiredCorrelogramFrameWidth = 512;
end

global gCorrelogramFrameRate
if exist('gCorrelogramFrameRate', 'var') && ~isempty(gCorrelogramFrameRate)
    desiredCorrelogramFrameRate = gCorrelogramFrameWidth;
else
    desiredCorrelogramFrameRate = 100;
end

global gCacheCorrelogramFile
if exist('gCacheCorrelogramFile', 'var') && ~isempty(gCacheCorrelogramFile)
    cacheCorrelogramFile = gCacheCorrelogramFile;
else
    cacheCorrelogramFile = 1;
end
debug = 0;

returnCorrelograms = nargout > 0;

%%
if 0
    % Code to check that correlograms and pitch data have the same sizes.
    % Just for debugging!
    for fileNumber = indexList
        correlogramFile = fileList(fileNumber).correlogramFile;
        pitchData = fileList(fileNumber).pitchData;
        if exist(correlogramFile, 'file')
            cd = load(correlogramFile);
            correlogramData = cd.correlogramData;
            if size(correlogramData, 2) ~= size(pitchData, 2)
                error('Bad Sizes.');
            end
        else
            warning(sprintf('Can''t find correlogram data for %s.', fileStruct.waveFile));
        end
    end
    return;
end

%%
% Calculate the correlograms for these test files
if debug
    if length(indexList) > 1
        fprintf('ComputeCorrelograms: Checking correlograms for %d files.\n', ...
            length(indexList));
    else
          fprintf('ComputeCorrelograms: Checking correlogram of %s.\n', ...
            fileList(indexList(1)).waveFile);
    end
end

for fileNumber = indexList
    if ~forceComputation && ~isempty(fileList(fileNumber).correlogram)
        continue;
    end
    
    correlogramFile = fileList(fileNumber).correlogramFile;
    if ~forceComputation
        if ~isempty(correlogramFile) && exist(correlogramFile,'file')
            if returnCorrelograms
                [correlogramData, correlogramFrameRate, numChannels, correlogramFrameWidth] = ...
                    RestoreCorrelogram(fileList(fileNumber));
                fileList(fileNumber).correlogram = correlogramData;
                fileList(fileNumber).correlogramFrameRate = correlogramFrameRate;
                fileList(fileNumber).numChannels = numChannels;
                fileList(fileNumber).correlogramFrameWidth = correlogramFrameWidth;
            end
            continue;
        end
    end
    soundFile = fileList(fileNumber).waveFile;
    fprintf('ComputeCorrelogram: computing correlogram of file %d, %s\n', ...
        fileNumber, soundFile);
    [soundData, fs] = audioread(soundFile);
    % Lots of resampling fractions at:
    %  http://www.soundslogical.com/support/mpacks/documentation/english/documentparts/mpack1over.html#P330_20517
    sound16 = [];               % To keep parfor happy.
    switch fs
        case 8000
            sound16 = resample(soundData, 2, 1);
        case 16000
            sound16 = soundData;
        case 22050
            sound16 = resample(soundData, 320, 441);
        case 44100
            sound16 = resample(soundData, 160, 441);
        case 48000
            sound16 = resample(soundData, 1, 3);
        otherwise
            error('FindCorrelogramPCA: Unknown sound sample rate.');
    end
    fs16 = 16000;
    
    if exist('gSignalToNoiseRatio', 'var') && ~isempty(gSignalToNoiseRatio) ...
            && isfinite(gSignalToNoiseRatio)
        sound16 = AddNoise(sound16);
    end
    
    
    [correlogramData, correlogramFrameRate, numChannels] = ...
        PitchCorrelogram(sound16, fs16, desiredCorrelogramFrameRate, desiredCorrelogramFrameWidth);
    correlogramFrameWidth = desiredCorrelogramFrameWidth;
    pitchData = fileList(fileNumber).pitchData;
    if ~isempty(pitchData)
        frameCount = size(pitchData, 2);
        if size(correlogramData, 2) > frameCount
            correlogramData = correlogramData(:,1:frameCount);
            % fprintf(' Removing %d frames from correlogramData.\n', ...
            %     size(correlogramData, 2)-frameCount);
        elseif size(correlogramData, 2) < frameCount
            newFrameCount = frameCount - size(correlogramData, 2);
            newFrameSize = size(correlogramData,1);
            correlogramData = [correlogramData zeros(newFrameSize, newFrameCount)];
            % fprintf(' Adding %d frames to correlogramData\n', newFrameCount);
        end
    end
    % Cache the result onto disk.
    if cacheCorrelogramFile
        mySave(correlogramFile, correlogramData, correlogramFrameRate, ...
            numChannels, correlogramFrameWidth, soundFile);
    end
    if returnCorrelograms
        fileList(fileNumber).correlogram = correlogramData;
        fileList(fileNumber).correlogramFrameRate = correlogramFrameRate;
        fileList(fileNumber).numChannels = numChannels;
        fileList(fileNumber).correlogramFrameWidth = correlogramFrameWidth;
    end
end

% Needed so that the parfor knows that this is the only stuff being saved.
function mySave(correlogramFile, correlogramData, correlogramFrameRate, ...
    numChannels, correlogramFrameWidth, soundFile)
save(correlogramFile, '-v7.3', 'correlogramData', 'correlogramFrameRate', ...
    'numChannels', 'correlogramFrameWidth', 'soundFile');



% We know that gSignalToNoiseRatio is set before getting here.
function noisySignal = AddNoise(rawSignal)
global gSignalToNoiseRatio gNoiseType

snrLevel = 10.^(gSignalToNoiseRatio/20);
signalLevel = std(rawSignal);
noiseLevel = signalLevel/snrLevel;

if exist('gNoiseType','var') && strcmp(gNoiseType, 'pink')
    % Pink noise generated by Julius
    %   https://ccrma.stanford.edu/~jos/sasp/Example_Synthesis_1_F_Noise.html
    Nx = length(rawSignal);     % number of samples to synthesize
    B = [0.049922035 -0.095993537 0.050612699 -0.004408786];
    A = [1 -2.494956002   2.017265875  -0.522189400];
    nT60 = round(log(1000)/(1-max(abs(roots(A))))); % T60 est.
    v = randn(Nx+nT60,1);       % Gaussian white noise: N(0,1)
    x = filter(B,A,v);          % Apply 1/F roll-off to PSD
    scaleFactor = 1/std(x(nT60+1:end));
    noise = scaleFactor * noiseLevel * x(nT60+1:end);      % Skip transient response
else
    noise = noiseLevel * randn(size(rawSignal));
end

noisySignal = rawSignal + noise;
fprintf('Using an SNR of %g: signal level %g, noise level %g.\n', ...
    gSignalToNoiseRatio, signalLevel, noiseLevel);
