function [mlpModel, flog] = ComputeMLPPitchModel(fileList, pcaModel, ...
    hiddenUnitCountList, indexList) 
% COMPUTEMLPPITCHMODEL Create the multilayer perceptron to predict pitch
%
% function [mlpModel, flog] = ComputeMLPPitchModel(fileList, pcaModel, ...
%   hiddenUnitCountList, indexList)
% Find a good MLP network to transform (PCA-rotated) correlogram data into
% pitch class.
%
% This routine uses the correlogram frames computed by the PCA calculation.
%  We need to find the ground truth pitch for these frames, and then we can
%  pass the PCA data and the pitch data to the MLP training routines.
%
% This routine can calculate the modelf for a range of hidden unit counts,
% but in the end only one model is returned (for 1000 hidden units).

if nargin < 3 || isempty(indexList)
    indexList = 1:length(fileList);
end

if nargin < 3 || isempty(hiddenUnitCountList)
    hiddenUnitCountList = 1000;
end

if ~exist('mlp', 'file')
    path(path, '../netlab');
end

%%
% Make sure the data from Graz and the
% correlogram data are about the same length.

if 0
    for i=1:length(fileList)
        correlogramData = fileList(i).correlogram;
        if isempty(correlogramData), continue; end
        pitchData = fileList(i).pitchData;
        [ph, pw] = size(pitchData);
        [ch, cw] = size(correlogramData);
        if abs(pw-cw) > 6
            fprintf('Pitch data for file %d has %d frames, correlogram has %d frames.\n', ...
                i, pw, cw);
            fprintf('\tFilename is %s.\n', fileList(i).pitchFile);
        end
    end
end
%%

% Assemble the MLP input and labels.  The input data is PCA-transformed
% correlogram data.  The labels are quantized pitch classes.
if 0            % Get all the data!
    % Compute the PCA transform of the correlogram data.
    for fileNumber = indexList
        [correlogramData, correlogramFrameRate, numChannels, correlogramFrameWidth] ...
            = RestoreCorrelogram(fileList(fileNumber));
        fileList(fileNumber).pcaCorrelogram = PCATransform(correlogramData, pcaModel);
    end
    % Concatenate all the PCA'ed correlogram data into one big array.
    pcaCorrelogram = cat(2, fileList(indexList).pcaCorrelogram);


    %%
    % Now look at the histogram of the pitch (ground truth) data
    pitchClass = cat(2, fileList(indexList).pitchClass);
else                % Just take a sample of the data.        
    allFrameCount = 0;
    for fileNumber = indexList
        allFrameCount = allFrameCount + length(fileList(fileNumber).pitchClass);
    end
        if isempty(fileList(fileNumber).correlogram)
            [correlogramData] = RestoreCorrelogram(fileList(i));
        else
            correlogramData = fileList(fileNumber).correlogram;
        end
    pcaCorrelogram = PCATransform(correlogramData, pcaModel);
    pcaSize = size(pcaCorrelogram, 1);
    
    maxMemory = 3e9;
    maxFrames = maxMemory/8/pcaSize;
    framesPerFile = round(maxFrames/length(indexList));
    
    allFrameCount = 0;
    for fileNumber = indexList
        thisFrameCount = length(fileList(fileNumber).pitchClass);
        frameIndices = randperm(thisFrameCount);
        frameIndices = frameIndices(1:min(length(frameIndices), framesPerFile));
        allFrameCount = allFrameCount + length(frameIndices);
        frameFlags = zeros(1, thisFrameCount);
        frameFlags(frameIndices) = 1;
        fileList(fileNumber).highEnergy = frameFlags;
    end
    pcaCorrelogram = zeros(pcaSize, allFrameCount);
    pitchClass = zeros(1, allFrameCount);
    allFramePointer = 1;
    
    for fileNumber = indexList
        frameFlags = logical(fileList(fileNumber).highEnergy);
        thisFrameCount = sum(frameFlags);
        if isempty(fileList(fileNumber).correlogram)
            [correlogramData] = RestoreCorrelogram(fileList(i));
        else
            correlogramData = fileList(fileNumber).correlogram;
        end
        myPCACorrelogram = PCATransform(correlogramData, pcaModel);
        pcaCorrelogram(:, allFramePointer:allFramePointer+thisFrameCount-1) = ...
            myPCACorrelogram(:, frameFlags);
        pitchClass(1, allFramePointer:allFramePointer+thisFrameCount-1) = ...
            fileList(fileNumber).pitchClass(frameFlags);
        allFramePointer = allFramePointer + sum(frameFlags);
    end
end
        
    
clf
hist(cat(2, fileList(indexList).pitchData), 100);
xlabel('Pitch'); ylabel('Frequency');
title('Histogram of pitch samples for MLP Test');

%%
% Create one-of-n label array so MLP knows what output is desired.
nClasses = max(QuantizePitches(1:1000));            % How many classes do we have
pitchProb = zeros(nClasses, size(pitchClass,2));    % Create 1-of-n array for output
for i=1:length(pitchClass)
    if isnan(pitchClass(i))
        pitchProb(1,i) = 1;                         % Assume unvoiced
    else
        pitchProb(pitchClass(i),i) = 1;
    end
end

%%

% Now train the MLP, trying a range of different hidden unit sizes.
splitCount = round(size(pcaCorrelogram, 2)*.75);
figure(1);
for i = 1:length(hiddenUnitCountList)
    perms = randperm(size(pcaCorrelogram, 2));
    trainingIndices = perms(1:splitCount);
    testIndices = perms(splitCount+1:end);
    trainingData = pcaCorrelogram(:,trainingIndices);
    testData = pcaCorrelogram(:,testIndices);
    
    outputData = pitchProb(:,trainingIndices);
    testOutput = pitchProb(:,testIndices);
    
    nHidden = hiddenUnitCountList(i);
    [mlpModel, meanGood, meanBad, flog] = OptimizePitchMLP(trainingData, outputData, ...
        testData, testOutput, nHidden);
    mlpGood(i) = meanGood;
    mlpBad(i) = meanBad;
end

%%
if length(hiddenUnitCountList) > 1
    figure(2);
    plot(hiddenUnitCountList, mlpGood./mlpBad);
    xlabel('Number of Hidden Units');
    ylabel('Mean Good Label / Mean Bad Label');
    title('Performance of MLP for Translating Correlogram to Pitch');
end

%%
% Now just do the 1000 hidden layer net so we can use it in further
% experiments.  It did well enough in early testing.
if length(hiddenUnitCountList) > 1
    perms = randperm(size(pcaCorrelogram, 2));
    trainingIndices = perms(1:splitCount);
    testIndices = perms(splitCount+1:end);
    trainingData = pcaCorrelogram(:,trainingIndices);
    testData = pcaCorrelogram(:,testIndices);
    
    outputData = pitchProb(:,trainingIndices);
    testOutput = pitchProb(:,testIndices);
    
    nHidden = 1000;
    [mlpModel, finalMeanGood, finalMeanBad, flog] = ...
        OptimizePitchMLP(trainingData, outputData, testData, testOutput, nHidden);
end

