function pcaModel = ComputePCACorrelogramModel(fileList, minEnergyThresholdFraction, indexList, pcaType)
% COMPUTEPCACORRELOGRAMMODEL Use PCA to summarize correlogram data
%
% function pcaModel = ComputePCACorrelogramModel(fileList, minEnergyThresholdFraction, indexList, pcaType)
% Given a collection of pitch correlograms, find the PCA model that best
% approximates each correlogram frame.  fileList contains the list of files
% (and their correlograms).  minEnergyThreshold is used to find the
% correlograms with sound.  Frames with less energy than
% minEnergyThreshold*mean(frameEnergy) are considered silence.  indexList
% is the list of fileList indices to use for trainning this model.  pcaType
% is a Netlab parameter.


if nargin < 2
    minEnergyThresholdFraction = 0.1;
end

if nargin < 3
    indexList = 1:length(fileList);
end

if nargin < 4
    pcaType = 'channel';
end

%%

if 0
    pcaFileNumbersToTest = randperm(length(fileList));
    count = max(1, pcaFilesToTest);
    pcaFileNumbersToTest = pcaFileNumbersToTest(1:count);
end

%%
% Check to make sure we have all the right sound files too.
if 0
    newFileList = [];
    for i=1:length(pcaFileNumbersToTest)
        fileNumber = pcaFileNumbersToTest(i);
        if exist(fileList(fileNumber).waveFile, 'file')
            newFileList = [newFileList fileNumber];
        end
    end
    pcaFileNumbersToTest = newFileList;
end

%%
% Grab the energy profiles for these test files.
for fileNumber = indexList
    if isempty(fileList(fileNumber).correlogram)
        [correlogramData] = RestoreCorrelogram(fileList(fileNumber));
    else
        correlogramData = fileList(fileNumber).correlogram;
    end
    fileList(fileNumber).correlogramEnergy = sum(correlogramData.^2, 1);
    frameCount = size(correlogramData, 2);
    fileList(fileNumber).correlogramFrameCount = frameCount;
end

%%
corEnergy = cat(2, fileList.correlogramEnergy);
[histCount,histCenter] = hist(corEnergy,1000);
if 1        % Old way, based on desired number of frames
    integratedCount = cumsum(histCount);
    if 0
        pcaNumberOfTrainingFrames = 3000;
    else
        pcaNumberOfTrainingFrames = floor(6e9/8/size(correlogramData,1));
    end
    pcaCountThresh = length(corEnergy) - 1.0 * pcaNumberOfTrainingFrames;
    desiredBin = max(2,find(integratedCount > pcaCountThresh, 1)-1);
    correlogramEnergyThreshold = histCenter(desiredBin);
else        % New way based on energy fraction
    meanEnergy = mean(corEnergy);
    correlogramEnergyThreshold = meanEnergy * minEnergyThresholdFraction;
end
    
%%
clf
loglog((histCenter), histCount/max(histCount), ...
    [(correlogramEnergyThreshold) (correlogramEnergyThreshold)], [1e-5 1]);
xlabel('Energy of Correlogram Frame');
ylabel('Fraction of Frames with this Energy');
title('Histogram of Correlogram Frame Energies');
legend('Correlogram Energy Histogram', 'PCA Threshold');
drawnow;
%%
if strcmp(pcaType, 'image')
    pcaModel = ComputeImagePCA(fileList, indexList, ...
        corEnergy, correlogramEnergyThreshold);
else
    pcaModel = ComputeChannelPCA(fileList, indexList, ...
        corEnergy, correlogramEnergyThreshold);
end

pcaModel.meanEnergy = mean(corEnergy);
pcaModel.minEnergyThresholdFraction = minEnergyThresholdFraction;

function pcaModel = ComputeImagePCA(fileList, indexList, corEnergy, correlogramEnergyThreshold)
% Count out how many correlogram frames we have to work with.
goodCorrelogramFrames = corEnergy > correlogramEnergyThreshold;
numExamples = sum(goodCorrelogramFrames);
pcaInputData = [];
pcaInputP = 1;
% Go through all the files and pick out just the frames that have enough
% energy to be interesring. Create an array with the data we want to
% summarize with PCA.
for i=indexList
    biggerList = find(fileList(i).correlogramEnergy > correlogramEnergyThreshold);
    if isempty(biggerList)
        continue;
    end
    nFrames = length(biggerList);
    if isempty(fileList(fileNumber).correlogram)
        [correlogramData, ~, ~, correlogramFrameWidth] = RestoreCorrelogram(fileList(i));
    else
        correlogramData = fileList(fileNumber).correlogram;
        correlogramFrameWidth = fileList(fileNumber).correlogramFrameWidth;
    end
    
    newPiece = correlogramData(:, biggerList);
    if isempty(pcaInputData)
        pcaInputData = zeros(size(correlogramData,1), numExamples);
    end
    pcaInputData(:, pcaInputP:pcaInputP+nFrames-1) = newPiece;
    fileList(i).highEnergy = zeros(1, length(fileList(i).correlogramEnergy));
    fileList(i).highEnergy(biggerList) = 1;
    pcaInputP = pcaInputP + nFrames;
end


%%
% Now do the PCA.
fprintf('Computing the principal components of a %d x %d matrix\n', ...
    size(pcaInputData,1), size(pcaInputData,2));
[pcaVectors, pcaEigenValues, pcaMeanValues] = ComputePCA(pcaInputData);

%%
figure(1); clf
semilogy(max(0,pcaEigenValues));
xlabel('Eigenvalue Number'); ylabel('Magnitude'); 
title('Eigenvalues for Correlogram PCA');
c = min(1000, length(pcaEigenValues));
axis([0 c pcaEigenValues(c) 1])
drawnow;

%%
% Show the first correlogram eigenvalues
figure(2); clf
frameChannels = size(pcaInputData,1)/correlogramFrameWidth;
for i=1:min(100, size(pcaVectors, 2));
    frame=reshape(pcaVectors(:,i),frameChannels,correlogramFrameWidth);
    imagesc(frame);
    title(sprintf('Eigenvalue %d', i));
    drawnow;
    pause(0.0);
end

%%
% Show the first 16 on a plot.
figure(3);
clf;
for i=1:16
    subplot(4,4,i);
    frame=reshape(pcaVectors(:,i),frameChannels,correlogramFrameWidth);
    imagesc(frame);
end
drawnow;

pcaModel = struct('pcaType', 'image', ...
    'pcaVectors', pcaVectors, ...
    'pcaEigenValues', pcaEigenValues, ...
    'pcaMeanValues', pcaMeanValues, ...
    'nPCA', [], ...
    'correlogramEnergyThreshold', correlogramEnergyThreshold, ...
    'meanEnergy', 0, ...
    'minEnergyThresholdFraction', 0);


%%
%           pcaVectors: [256x256x40 double] frameWidth x nPCA x nChannels
%       pcaEigenValues: [256x40 double]     nPCA x nChannels
%        pcaMeanValues: [256x40 double]     frameWidth x nChannels

function pcaModel = ComputeChannelPCA(fileList, indexList, ...
    corEnergy, correlogramEnergyThreshold)
% Count out how many correlogram frames we have to work with.
goodCorrelogramFrames = corEnergy > correlogramEnergyThreshold;
numExamples = sum(goodCorrelogramFrames);
pcaInputData = [];
for c=1:200             % Arbitrary upper limit until we find out true value
    % Go through all the files and pick out just the frames that have enough
    % energy to be interesring. Create an array with the data we want to
    % summarize with PCA.
    pcaInputP = 1;    
    for i=indexList
        biggerList = find(fileList(i).correlogramEnergy > correlogramEnergyThreshold);
        if isempty(biggerList)
            continue;
        end
        nFrames = length(biggerList);
        if isempty(fileList(i).correlogram)
            [correlogramData, ~, numChannels, correlogramFrameWidth] = RestoreCorrelogram(fileList(i));
        else
            correlogramData = fileList(i).correlogram;
            correlogramFrameWidth = fileList(i).correlogramFrameWidth;
            numChannels = size(correlogramData,1) / correlogramFrameWidth;
        end
        channelImage = reshape(correlogramData(:, biggerList), numChannels, ...
            correlogramFrameWidth, []);
        newChannelData = squeeze(channelImage(c, :, :));
        if isempty(pcaInputData)
            pcaInputData = zeros(correlogramFrameWidth, numExamples);
            modelEigenVectors = zeros(correlogramFrameWidth, correlogramFrameWidth, numChannels);
            modelEigenValues = zeros(correlogramFrameWidth, numChannels);
            modelMeanValues = zeros(correlogramFrameWidth, numChannels);
        end
        pcaInputData(:, pcaInputP:pcaInputP+nFrames-1) = newChannelData;
        pcaInputP = pcaInputP + nFrames;
    end
    if 1
        imagesc(pcaInputData); colorbar; 
        title(sprintf('Channel %d PCA Input', c));
        drawnow;
    end
    % Now do the PCA.
    fprintf('Channel %d: Computing the principal components of a %d x %d matrix\n', ...
        c, size(pcaInputData,1), size(pcaInputData,2));
    if 1
        [pcaVectors, pcaEigenValues, pcaMeanValues] = ComputePCA(pcaInputData);
    else                    % Use identify matrix as result for 
        fprintf('Forcing PCA to be identity matrix!!!\n');
        numElements = size(pcaInputData, 1);
        pcaVectors = eye(numElements);
        pcaEigenValues = ones(numElements, 1);
        pcaMeanValues = zeros(numElements, 1);
    end
    % Store the channel by channel results
    modelEigenVectors(:,:,c) = pcaVectors;
    modelEigenValues(:,c) = pcaEigenValues(:);
    modelMeanValues(:,c) = pcaMeanValues(:);
    fileList(i).highEnergy = zeros(1, length(fileList(i).correlogramEnergy));
    fileList(i).highEnergy(biggerList) = 1;
    if c >= numChannels          % Didn't know number of channels until we read a correlogram in
        break
    end
end

%%


pcaModel = struct('pcaType', 'channel', ...
    'pcaVectors', modelEigenVectors, ...
    'pcaEigenValues', modelEigenValues, ...
    'pcaMeanValues', modelMeanValues, ...
    'nPCA', [], ...
    'correlogramEnergyThreshold', correlogramEnergyThreshold, ...
    'meanEnergy', 0, ...
    'minEnergyThresholdFraction', 0);

%%
figure(1);
semilogy(pcaModel.pcaEigenValues)
xlabel('EigenValue Number'); ylabel('Value');
title('EigenValues for each channel model');

figure(2);
plot(mean(pcaModel.pcaEigenValues))
xlabel('Channel Number');
ylabel('Mean EigenValue Value');
title('Channel Variation vs. Cochlear Filtebank Channel');