function [pitchModel, fileList] = ComputePitchModel(datasetName, pcaThresh, ...
    hiddenUnitCountList, indexList)
% COMPUTEPITCHMODEL Create an entire pitch model (correlograms, pca, mlp)
%
% function [pitchModel, fileList] = ComputePitchModel(datasetName, pcaThresh, ...
%    hiddenUnitCountList, indexList)
% Do all the calculations necessary to create the entire pitch model.  This
% routine starts with a database name and creates the PCA and MLP models
% that predict the pitch of a sound.  
% 
% There are four parameters:
% 1) the name of the database we use for training.  This is one of the
%   valid arguments for the FindPitchFiles routine.
% 2) pcaThresh - the energy threshold that determines which correlogram
%   frames are used for the PCA calculation.  This parameter is a function of
%   the mean energy of each frame.  Values > 1 are typical.
% 3) hiddenUnitCountList - How many hidden units to use when designing the
%    MLP.
% 4) indexList - a list of indices from the fileList to use for training.
%   This is used to cut down the training set size.
% 
% This routine returns two structures.
% 1) the pitchModel structure, which gives the CalculatePitch routine all
%   the information it needs to calculate a pitch.
% 2) the fileList structure, which contains information about the training
%   files.
%%
if nargin < 2 || isempty(pcaThresh);
    pcaThresh = 2.0;
end

if nargin < 3 || isempty(hiddenUnitCountList)
    hiddenUnitCountList = [1 2 4 6 8 10 12 14 16 20 24]*100;
    hiddenUnitCountList = 1000;
end

if nargin < 4
    indexList = [];
end

if length(pcaThresh(:)) > 1
    error('ComputePitchModel: pcaThresh must be a scalar.');
end
%%
%  hiddenUnitCountList = 1000; pcaThresh = 4; indexList = [];  datasetName = 'keele';

fileList = FindPitchFiles(datasetName);

if isempty(indexList), indexList = 1:length(fileList); end

%%
global gCorrelogramFrameWidth;
if exist('gCorrelogramFrameWidth', 'var') && ~isempty(gCorrelogramFrameWidth)
    gCorrelogramFrameWidth = 512;
end

fprintf('Finding pitch transition probabilities for %d files.\n', length(indexList));
figure(1);
[transitionProbabilities, classProbabilities, allTransitions] =  FindTransitionProbabilities(fileList);

%%

global gForceCorrelogramCalculation
if exist('gForceCorrelogramCalculation', 'var') && ~isempty(gForceCorrelogramCalculation)
    forceCorrelogramCalculation = gForceCorrelogramCalculation;
else
    forceCorrelogramCalculation = 0;
end

global gMaximumFileCount
if exist('gMaximumFileCount', 'var') && ~isempty(gMaximumFileCount)
    maximumFileCount = gMaximumFileCount;
else
    maximumFileCount = 0;
end
if maximumFileCount > 0
    indexList = indexList(1:min(maximumFileCount,length(indexList)));
end

fprintf('Computing correlogram features for %d sounds.....\n', length(indexList));
fileList = ComputeCorrelograms(fileList, indexList, forceCorrelogramCalculation);

%%
fprintf('Finding PCA model with a energy threshold of %d x Mean Energy\n', ...
    pcaThresh);
figure(2);
pcaModel = ComputePCACorrelogramModel(fileList, pcaThresh, indexList);

global gPitchPCATerms
if exist('gPitchPCATerms', 'var') && ~isempty(gPitchPCATerms)
	pcaModel.nPCA = gPitchPCATerms;
else
    pcaModel.nPCA = 20;
end
% Get rid of superfluous vectors--Don't need to store values we won't use.
pcaModel.pcaVectors = pcaModel.pcaVectors(:,1:pcaModel.nPCA,:);


%%
fprintf('Finding the MLP model to transform correlogram into pitch probs.\n');
[mlpModel,mlpLog] = ComputeMLPPitchModel(fileList, pcaModel, hiddenUnitCountList, indexList);

%%
pitchModel = struct(...
    'datasetName', datasetName, ...
    'trainingFiles', [], ...
    'trainingList', indexList, ...
    'pcaModel', pcaModel, ...
    'mlpModel', mlpModel, ...
    'mlpEvalLog', mlpLog, ...
    'transitionProbabilities', transitionProbabilities, ...
    'classProbabilities', classProbabilities, ...
    'allTransitions', allTransitions, ...
    'lambda', 1.0, ...
    'pcaThresh', pcaThresh, ...
    'correlogramFrameWidth', gCorrelogramFrameWidth, ...
    'classRegularlizer', 1.0);

% Do separate so we don't get an array of structs.  If we put this above we
% get an array. :-(
pitchModel.trainingFiles = {fileList.waveFile}; 

allPitchData = cat(2, fileList(indexList).pitchData); 
[pitchHist, pitchHistCenters] = hist(allPitchData(allPitchData>10), 100);
pitchModel.pitchHist = pitchHist;
pitchModel.pitchHistCenters = pitchHistCenters;

return;

%%
% Test code...  Plot summary statistics about the data we used and the
% model we've just designed.
if strcmp(pitchModel.pcaModel.pcaType, 'image')
    figure(2);
    semilogy(max(0,pitchModel.pcaModel.pcaEigenValues));
    xlabel('Eigenvalue Number'); ylabel('Magnitude'); 
    title('Eigenvalues for Correlogram PCA');
    c = min(1000, length(pitchModel.pcaModel.pcaEigenValues));
    axis([0 c pitchModel.pcaModel.pcaEigenValues(c) 1])
else
    figure(2); clf
    semilogy(pitchModel.pcaModel.pcaEigenValues)
    xlabel('EigenValue Number'); ylabel('Value');
    title('EigenValues for each channel model');

    figure(3); clf
    plot(mean(pitchModel.pcaModel.pcaEigenValues))
    xlabel('Channel Number');
    ylabel('Mean EigenValue Value');
    title('Channel Variation vs. Cochlear Filtebank Channel');
end
figure(4); clf
imagesc(pitchModel.pcaModel.pcaVectors);
xlabel('Vector');
ylabel('Frame Data');

figure(5); clf
semilogy(pitchModel.mlpEvalLog)
xlabel('MLP Training Iteration Count');
ylabel('MLP Evaluation Function');
title('Pitch MLP Performance vs. Training Iterations');

figure(6); clf
plot(pitchModel.pitchHistCenters, pitchModel.pitchHist);
xlabel('Pitch (Hz)');
ylabel('Frequency (counts)');
title('Voiced Pitch Distribution in Training and Testing Data');

figure(7); clf
imagesc(pitchModel.transitionProbabilities.^(1/10));
xlabel('To Probability Class'); ylabel('From Probability Class');
title('Squeezed Pitch Class Transition Probabilities');

%%
if ~exist('savefigure', 'file')
     path(path,'..\SaveFigure');
end

for i=1:7
    figure(i);
    fn = sprintf('PitchModel-%d.png', i);
    savefigure(fn, 'FontSize',0.25, 'MinFontSize', 5);
end

