function [fileList] = FindPitchFiles(database)
% FINDPITCHFILES Look for all sound (and pitch) files in a database.
%
% function [fileList] = FindPitchFiles(database)
% Find all the pitch files (and corresponding WAV files) that go with a
% particular database of pitch data.
%%
if nargin < 1 || isempty(database)
    database = 'Graz';
end

%%
defaultPitchDBDir = '../PitchTrackingDatabase/';
if ~exist(defaultPitchDBDir, 'dir')
    defaultPitchDBDir = ['../' defaultPitchDBDir];
    if ~exist(defaultPitchDBDir, 'dir')
       error('Cannot find PitchTrackingDatabase');
    end
end

switch lower(database)
    case 'graz'
        pitchSuffix = '.f0';
        pitchDirectory = [defaultPitchDBDir 'Graz'];
    case {'keeleextended', 'keele'}
        pitchSuffix = '.gth';
        pitchDirectory = [defaultPitchDBDir 'keele'];
    case 'fda'
        pitchSuffix = '.gth';
        pitchDirectory = [defaultPitchDBDir 'fda'];
    case 'test'
        pitchSuffix = '.f0';
        pitchDirectory = [defaultPitchDBDir 'Test'];
    case 'liz'
        pitchSuffix = '.ascii';
        pitchDirectory = [defaultPitchDBDir 'LizSamples'];
    case 'auroratrainclean'
        pitchSuffix = '.WAV';           % No pitch, just wav.. fix later.
        pitchDirectory = '\\speechstore5\corpora\Aurora4\PCM\16k\train_clean';
    case 'auroratestclean'
        pitchSuffix = '.WAV';           % No pitch, just wav.. fix later.
        pitchDirectory = '\\speechstore5\corpora\Aurora4\PCM\16k\test_clean_wv1';
    otherwise
        error('FindTransitionProbabilities: Unknown database');
end
        
fileNameList = FindFiles(pitchDirectory, pitchSuffix);

%%
if strcmpi(database, 'keele')
    i=1;
    while i <= length(fileNameList)
        finds = strfind(fileNameList(i), '0000Pitch');
        if ~isempty(finds{1})
            fileNameList(i) = [];
        else
            i = i + 1;
        end
    end
end
%%
if isempty(fileNameList)
    fprintf('Couldn''t find any files matching the suffix %s in %s.\n', ...
        pitchSuffix, pitchDirectory);
    fileList = [];
    return;
end

clear fileList

fileList(length(fileNameList)) = struct(...
    'pitchFile', [], ...
    'waveFile', [], ...
    'pitchData', [], ...
    'pitchClass', [], ...
    'pitchRate', 0, ...
    'highEnergy', [], ...
    'correlogram', [], ...
    'correlogramFile', [], ...
    'correlogramEnergy', [], ...
    'numChannels', [], ...
    'correlogramFrameRate', [], ...
    'correlogramFrameWidth', [], ...
    'correlogramFrameCount', 0, ...
    'pcaCorrelogram', [], ...
    'mlpCorrelogram', [] ...
);

for i=1:length(fileNameList)
    fileList(i).pitchFile = fileNameList{i};
    switch lower(database)
        case 'graz'
            soundFile = strrep(fileList(i).pitchFile, '/REF/', '/MIC/');
            soundFile = strrep(soundFile, '/ref_', '/mic_');
            soundFile = strrep(soundFile, '.f0', '.wav');
            pitchRate = 100;
        case {'keele', 'fda', 'keeleextended'}
            soundFile = strrep(fileList(i).pitchFile, '/gth/', '/wav/');
            soundFile = strrep(soundFile, '.gth', '.wav');
            pitchRate = 100;
        case 'test'
            soundFile = strrep(fileList(i).pitchFile, '.f0', '.wav');
            pitchRate = 1000;
        case 'liz'
            soundFile = strrep(fileList(i).pitchFile, '.ascii', '.wav');
            pitchRate = 100;
        case {'auroratrainclean', 'auroratestclean'}
            soundFile = fileList(i).pitchFile;
            fileList(i).pitchFile = [];
            pitchRate = 100;
        otherwise
            error('FindTransitionProbabilities: Unknown database');
    end
    fileList(i).waveFile = soundFile;
    fileList(i).pitchRate = pitchRate;
    if 1
        fileList(i).correlogramFile = strrep(soundFile, '.wav', '.mat');
    else
        basename = strrep(soundFile, pitchDirectory, '');
        basename = strrep(basename, '.WAV', '');
        basename = strrep(basename, '.wav', '');
        basename = strrep(basename, '\\', '_');
        basename = strrep(basename, '/', '_');
        matFile = sprintf('%s\\%s.mat', '\\localhost\PitchTrackingScratch', basename);
        fileList(i).correlogramFile = matFile;
    end
    if ~isempty(fileList(i).pitchFile)
        pitchReference = load(fileList(i).pitchFile);
        % Hack attack.. on keele data files 101-110 seem to have negative pitch
        % values... take absolute value.... NOW just set the data to nan since
        % he isn't sure of the ground truth.
        % Pitch == 0 means unvoiced
        % Pitch == nan means no judgement
        fileList(i).pitchData = (pitchReference(:,1)');
        fileList(i).pitchData(fileList(i).pitchData < 0) = nan;
        fileList(i).pitchClass = QuantizePitches(fileList(i).pitchData);
        fileList(i).correlogramFrameCount = 0;
    end
end



function fileList = FindFiles(initialDir, pattern)
fprintf('FindFiles: Checking %s for %s\n', initialDir, pattern);
fileList = {};
dirListing = dir(initialDir);
for i = 1:length(dirListing)
    if strncmp(dirListing(i).name, '.', 1)
        continue;
    elseif dirListing(i).isdir
        newDir = [initialDir '/' dirListing(i).name];
        fileList = [fileList FindFiles(newDir, pattern)];
    elseif strfind(dirListing(i).name, pattern) >= 0
        newName = [initialDir '/' dirListing(i).name];
        fileList = [fileList {newName}];
    end
end
