function [net, meanGood, meanBad, flog, testPredictions] = ...
    OptimizePitchMLP(trainingData, trainingOutput, testData, testOutput, nHidden)
% function [net, meanGood, meanBad, testPredictions] = ...
%   MakePitchMLP(trainingData, trainingOutput, testData, testOutput, nHidden)
% Create an MLP that transforms (PCA-transformed) coorrelogram data into
% pitch-class estimates. This routine takes training and testing data and
% outputs a network (from Netlab) and performance numbers.
%
% All observations are column vectors.. time is the second variable.
% But the Netlab tools assume that each observation is a row, so everything
% gets transposed before sending it to netlab.

nIn = size(trainingData, 1);			% Number of inputs.
nOut = size(trainingOutput, 1);              % Number of outputs.
alpha = 0.01;                           % Coefficient of weight-decay prior. 

if nargin < 5 || isempty(nHidden)
    nHidden = 2*nIn;                        % Number of hidden units.
end

global gMaximumMLPIterations
if exist('gMaximumMLPIterations', 'var') && ~isempty(gMaximumMLPIterations)
    mlpIterations = gMaximumMLPIterations;
else
    mlpIterations = 100;
end

%%

net = mlp(nIn, nHidden, nOut, 'logistic', alpha);

options = zeros(1,18);
options(1) = 1;                     % This provides display of error values.
options(14) = mlpIterations;		% Number of training cycles. 

% Train using scaled conjugate gradients.
[net, options, flog] = netopt(net, options, trainingData', trainingOutput', 'scg');

%%

trainingPredictions = mlpfwd(net, trainingData');

[~,trainingClass] = max(trainingOutput,[], 1);
[~,i] = sort(trainingClass);

subplot(2,1,1);
imagesc(trainingPredictions(i,:)'); colorbar;
title(sprintf('Training Results with %d (%s) hidden units', nHidden, net.outfn));
xlabel('Sorted Sample Number');
ylabel('Pitch Class');

testPredictions = mlpfwd(net, testData');

[~,testClass] = max(testOutput,[], 1);
[~,i] = sort(testClass);

subplot(2,1,2);
imagesc(testPredictions(i,:)'); colorbar;
title(sprintf('Testing Results with %d (%s) hidden units', nHidden, net.outfn));
xlabel('Sorted Sample Number');
ylabel('Pitch Class');

drawnow;

meanGood =  mean(max(testOutput.*testPredictions'));
meanBad = mean(mean((1-testOutput).*testPredictions'));
fprintf('Good responses average %g, bad response average %g, ratio is %g.\n', ...
    meanGood, meanBad, meanGood/meanBad);

%%
