function [deltaPitch, pitchChanges] = PitchChange(probs, smoothWindowLength, debug)
% PITCHCHANGE Calculate the pitch change from pitch probabities
%
% function [deltaPitch, pitchChanges] = PitchChange(probs, smoothWindowLength, debug)
% Look at the pitch-class probabilities and calculate the likely change in
% pitch. Use the smoothWindowLength to calculate how many frames we average
% before looking for the correlation changes.

if nargin < 1
    load tmp/ll/ll0006.probs
    probs = ll0006;
end
if nargin < 2 || isempty(smoothWindowLength)
    smoothWindowLength = 5;
end
if nargin < 3 || isempty(debug)
    debug = 1;
end
nFrames = size(probs, 2);

%%
if debug
    subplot(3,1,1);
    plot(probs(1,:))
    title('Pitch from MLP-Correlogram');
    ylabel('Pitch (Hz)');
    xlim([1 nFrames]);

    subplot(3,1,2);
    imagesc(probs(2:end,:))
    title('Correlogram Pitch Probabilities');
    ylabel('Pitch Class');
    xlim([1 nFrames]);
    axis xy
end

%%
% Probs data (from WebPitch) has the following rows first...
%   1: pitch, 2: unvoiced, 3:too low, 4: too high
% So start with row 5 to get just the pitch-class probabilities.
pitchClass = probs(5:end,:);
pitchSignalLength = 2*size(pitchClass, 1)-1;
pitchCenter = size(pitchClass,1);

pitchChanges = zeros(pitchSignalLength, size(pitchClass, 2)-1);
deltaPitch = zeros(1,size(pitchClass, 2)-1);

for curFrame = 1:size(pitchClass,2)-1
    c = xcorr(pitchClass(:,curFrame), pitchClass(:,curFrame+1));
    pitchChanges(:, curFrame) = flipud(c);
    deltaPitch(curFrame) = pitchCenter - FindMaxSuper(c);
end

% Apply a moving-average window to the raw pitch-change data.
if nargout > 1 && smoothWindowLength > 1
    radius = floor(smoothWindowLength/2);     % Half of total length
    rawPitchChanges = pitchChanges;
    for curFrame = 1:size(rawPitchChanges,2)
        b = max(1, curFrame - radius);
        e = min(size(rawPitchChanges, 2)-1, curFrame + radius);
        pitchChanges(:,curFrame) = mean(rawPitchChanges(:,b:e), 2);
    end
end

%%
if debug
    subplot(3,1,3);
    imagesc(1:nFrames-1, -pitchCenter+1:pitchCenter-1, pitchChanges);
    axis xy
    xlim([1.5 nFrames-0.5]);
    ylim([-4 4]);
    hold on
    plot(1.5:nFrames-0.5, deltaPitch, 'k-', [1 nFrames],[0 0], 'w:');
    hold off
    title('Pitch Change Signal')
    xlabel('Frame Number');
    ylabel('Pitch Change (classes/frame)');
end

function xmax = FindMaxSuper(y)
% Use super-resolution to find the location of the true maximum.  Fit the 
% data around the normal maximum with a 2nd degree polynomial.
[m,i] = max(y);
i = max(2,min(length(y)-1,i));
y0 = y(i);
ym = y(i-1);
yp = y(i+1);
xmax = i + 1/2*(yp-ym)/(-ym + 2*y0 - yp);
