 function [pitchTrackingError, voicedError, unvoicedError, bestOffset] = ...
    PitchTrackingError(groundTruth, pitchEstimate, offset, pObservations)
% PITCHTRACKINGERROR Calculate the pitch error using Lee & Ellis metric
% 
% function [pitchTrackingError, voicedError, unvoicedError, bestOffset] = ...
%     PitchTrackingError(groundTruth, pitchEstimate, offset, pObservations)
%
% Analyze the pitch tracking result and calculate the errors using the
% error measure proposed in
%   Lee and Ellis 2012, Noise Robust Pitch Tracking by Subband
%   Autocorrelation Classification, Interspeech 2012.
% This measure calculates a mean of two terms related to the error when
%	1) the groundtruth says the speech is voiced, and 
%   2) the error for unvoiced frames
% Input is the groundtruth (in Hz), the pitch estimate (in Hz), a vector of
% frame offsets to check (defaults to 0).  An optional fourth argument
% shows the pitch class observation probabilities and is only used for
% plotting the final result for this test.
% 
% Offset can be a single number, or a vector of potential offsets.  This
% routine returns the minimum error over the possible offsets.

if nargin < 3
    offset = 0;
end
if nargin < 4
    pObservations = [];
end

if length(offset) > 1
    % Handle the case if we have lots of different offsets (between ground
    % truth and pitch estimates). 
    minPTE = inf;
    minVE = inf;
    minUVE = inf;
    bestOffset = offset(1);
    for o = offset
        [pte, ve, uve] =  PitchTrackingError(groundTruth, pitchEstimate, o);
        if pte < minPTE
            minPTE = pte; minVE = ve; minUVE = uve; bestOffset = o;
        end
    end
    pitchTrackingError = minPTE; 
    voicedError = minVE;
    unvoicedError = minUVE;
    return
else
    bestOffset = offset;
end

% Now make sure that the arrays are the right size for easy comparison
if size(groundTruth,1) > size(groundTruth,2), groundTruth = groundTruth'; end;
if size(pitchEstimate,1) > size(pitchEstimate,2), pitchEstimate = pitchEstimate'; end;

if offset < 0
    groundTruth = groundTruth(-offset:end);
elseif offset > 0
    pitchEstimate = pitchEstimate(offset:end);
    if ~isempty(pObservations), pObservations = pObservations(:,offset:end); end
end
n = min(size(groundTruth,2), size(pitchEstimate,2));
groundTruth = groundTruth(1:n);
pitchEstimate = pitchEstimate(1:n);
if ~isempty(pObservations) pObservations = pObservations(:,1:n); end

% Compute bit vectors showing when two tracks are voiced/unvoiced.
groundTruthUnvoiced = logical(groundTruth < 2.0);
groundTruthVoiced = logical(groundTruth > 2.0);
pitchEstimateUnvoiced = logical(isnan(pitchEstimate) | pitchEstimate < 2.0);
pitchEstimateVoiced = ~pitchEstimateUnvoiced;

bothVoiced = groundTruthVoiced & pitchEstimateVoiced;

% Count up the number of voiced/unvoiced frames in the reference.  This
% becomes the denominators of the error metric.
numberVoiced = sum(groundTruthVoiced);
numberUnvoiced = sum(groundTruthUnvoiced);

Evu = sum(groundTruthVoiced & pitchEstimateUnvoiced);
Euv = sum(groundTruthUnvoiced & pitchEstimateVoiced);
relativeError = 0*groundTruth;
relativeError(bothVoiced) = (groundTruth(bothVoiced) - pitchEstimate(bothVoiced)) ./ ...
    groundTruth(bothVoiced);
Ef0 = sum(abs(relativeError) > 0.20);

voicedError = (Ef0 + Evu)/numberVoiced;
unvoicedError = Euv/numberUnvoiced;
pitchTrackingError = (voicedError + unvoicedError) / 2.0;

if ~isempty(pObservations)
    %%
    clf
    fVoicedButUnvoiced = groundTruthVoiced & pitchEstimateUnvoiced;
    fUnvoicedButVoiced = groundTruthUnvoiced & pitchEstimateVoiced;
    fNeither = ~(groundTruthVoiced | groundTruthUnvoiced);
    fPitchError = abs(relativeError) > 0.20;
    
    pseudoColor = 1+zeros(size(pObservations,1), size(pObservations,2), 3);
    pseudoColor(:,fVoicedButUnvoiced,1) = pseudoColor(:,fVoicedButUnvoiced,1) * .8;
    pseudoColor(:,fUnvoicedButVoiced,2) = pseudoColor(:,fUnvoicedButVoiced,2) * .8;
    pseudoColor(:,fNeither, 1:3) = pseudoColor(:,fNeither,1:3) * .8;
    pseudoColor(:,fPitchError,3) = pseudoColor(:,fPitchError,3) * .8;
    pseudoColor = pseudoColor .* repmat(1-sqrt(pObservations), [1,1,3]);
    
    image(pseudoColor)
    hold on;
    plot(QuantizePitches(groundTruth), 'x');
    plot(QuantizePitches(pitchEstimate), 'o');
    frameNumber = 0; frameStep = 400;
    axis([frameNumber*frameStep+1 (frameNumber+1)*frameStep-1 1 70]); frameNumber=frameNumber+1;
    axis xy;
    xlabel('Frame Number'); ylabel('Pitch Class'); 
    title('Pitch Tracking Summary (Blue: estimated unvoiced, Magenta: estimated voiced, Yellow: gross pitch)');
    legend('Ground Truth', 'Pitch Estimate');
    hold off
    %%
end