%% Pitch Change Demonstration Code
% This m-file demonstrates how to use the Pitch Change Toolkit to calculate
% how the pitch of a signal changes. Often a single value for the pitch 
% is hard to calculate due to octave ambiguities and other
% effects at the onset and offset of voicing. But the change in pitch is 
% usually what we care about, and this is easier to calculate.

%% Preliminaries
% We need both the Auditory Toolbox and Nabney's Netlab.  Make sure we have 
% the tools we need on our path.  Change these directory names to
% suit your own installation.
if ~exist('ERBFilterBank', 'file')
	path(path, '../../AuditoryToolbox');    % Add the Auditory Toolbox
end

if ~exist('mlpfwd', 'file')
    path(path, '../../Netlab');             % Add Netlab for machine learning
end
colormap('default');

%% Read audio and model for testing.
% Lets first test an exisiting pitch model. 
% Read in an audio file and the pitch model
% 

load grazPitchModel.mat

[inputSound,inputFs] = audioread('testSpeech.wav');

%% Calculate the necessary correlograms
% Next we calculate the correlograms of the sound. This requires a cochlear
% filter bank (gammatone) and the auto-correlation calculation.  The
% correlogram image is unwound and is stored as one column in the
% correlogramData file.
%
% We can then display one frame of the correlogram (the one frame with the
% most energy, which is voiced speech.)

frameRate = 100; 
frameWidth = 512;
[correlogramData, frameRate, numChannels] = PitchCorrelogram(inputSound, inputFs, frameRate, frameWidth);

if 1
    frameEnergy = sum(correlogramData.^2);
    [m,i] = max(frameEnergy);           % Find the most energetic frame for plotting
    clf; imagesc(reshape(correlogramData(:,i), [], frameWidth));
    xlabel('Autocorrelation Delay (tau in samples)');
    ylabel('Cochlear Channel');
    title('A Correlogram Frame');
end

%% Estimate the pitch probabilities of the test sound
% Given the correlogram, use PCA to reduce its dimensionality, and then use
% an MLP (multi-layer perceptron) to estimate the pitch-class probability at
% each time.
pcaPitch = PCATransform(correlogramData, pitchModel.pcaModel);

quantizedPitchEvidence = mlpfwd(pitchModel.mlpModel, pcaPitch')';

% The output is an estimate of the pitch class, as a function of time
% (frame number, 100 frames/second).
imagesc(quantizedPitchEvidence);
xlabel('Frame Number'); ylabel('Pitch Class'); title('Pitch Probabilities');
axis xy
% Pitch Class 1 (row 1 in the image below) is used to indicate unvoiced.

%% Estimate the pitch (Hz) at each time and plot the results.
% Use a Viterbi search to find the optimum tradeoff between observations,
% prior pitch probabilities, and the pitch-class transition probabilities.

fileStruct = struct('mlpCorrelogram', quantizedPitchEvidence);
[viterbiPitch,statePath,pObservations,pTransitions] = ...
    CalculateViterbiPitch(fileStruct, pitchModel);

%% Pitch Change
% Now estimate the pitch-change signal (by correlating neighboring 
% pitch-probability estimates) and plot the results. The pitch model
% estimates the likelihood of each possible pitch.  We correlate this
% vector (one dimension per pitch class) over time to get the pitch-change
% signal. In the plot below, the gray-scale image shows the pitch-change
% probability signal (computed by correlating over time).  While the blue
% line shows the "exact" position of the peak, computed by fitting the quadratic
% curve to the peak, and using super-resolution.
smoothWindowLength = 5;
debug = 0;
[deltaPitch, pitchChanges] = PitchChange(quantizedPitchEvidence, smoothWindowLength, debug);

clf; colormap(1-gray);
nT = floor(size(pitchChanges,1));
imagesc(1:size(pitchChanges,2), -nT:nT, pitchChanges);
hold on;
cleanPitchChange = deltaPitch;
cleanPitchChange(isnan(viterbiPitch)) = nan;
plot(1:size(cleanPitchChange,2), cleanPitchChange);
hold off;
xlabel('Frame Number'); ylabel('Pitch Change Signal'); 
ylim([-6 6])            % Only care about the center of this change signal

%% Compute a new pitch model
% The ComputePitchModel routine does all the necessary work to create a new
% pitch model.  It starts with a database name and does the following:
% 1) Compute all correlograms
% 2) Find the best PCA model to approximate each correlogram data
% 3) Reduces the dimensionality of the correlogram data
% 4) Find best MLP (multi-layer perceptron) to estimate the pitch class
% All the results are returned in a pitchModel struct, which stores all the
% necessary data so we can compute with this model.  
%
% The MLP training code splits the data so 75% is used for training and 
% 25% is used for testing. The training/testing results are sorted by their
% true pitch, so the correct result should be monotonic.
% 
% This step takes a while.
pcaThresh = 5;
databaseToUse = 'graz';
% We can't fit all the data into memory at once (to compute the PCA) so we
% subsample the training data.
fileList = FindPitchFiles(databaseToUse);
filesToUse = 1:10:length(fileList);         % Big training exercise
filesToUse = 1:100:length(fileList);        % Small training example
tic;
[pitchModel, fileList] = ComputePitchModel('graz', pcaThresh, [], filesToUse);
save newPitchModel pitchModel
toc;

%% Measure new pitch model's performance
% Measure performance of this model using a set of test data.  Most
% importantly, this function returns a graph that shows two things:
% 1) the effect of changing the importance of the transition probability
% matrix (larger, to the right, means prefer normal transitions), and 
% 2)the importance of the prior probabilities (different lines).
global gPlayPitchSounds
gPlayPitchSounds = 0;
tic;
filesToTest = 2:40:length(fileList);    % Select a different subset for testing.   
filesToTest = 2:100:length(fileList);   % Select a small subset for testing.   
[pitchTrackingError, voicedError, unvoicedError] = ...
    MeasurePitchPerformance(fileList(filesToTest), pitchModel);
toc;
%%
