function [bestPath,statePath,pObservations,pTransitions] = ...
    ViterbiDecoder(pObservations, pTransitions, lambda)
% VITERBIDECODER Generic Viterbi decoder given observations and transition probabilities
%
% function [bestPath,statePath,pObservations,pTransitions] = ...
%    ViterbiDecoder(pObservations, pTransitions, lambda)
% Find the best path combining observation data (nStates x nTimes) and
% state transition probabilities (nStates x nStates).  Trade off relative
% importance with lambda, which can be a list of allowable values.  Larger
% values of lambda weight the transition matrix more heavily.
if nargin < 2
    [pObservations, pTransitions] = TestViterbi();
end

if nargin < 3
    lambda = 1;
end

interactive = 0;

[nStates,nTimes] = size(pObservations);
[n1,n2] = size(pTransitions);
if n1 ~= n2 
    error('transitionMatrix is not square.');
    newpTransitions = zeros(nStates, nStates);
    newpTransitions(1:n1,1:n2) = pTransitions;
    pTransitions = newpTransitions;
elseif nStates > n1
    warning('More transitions than observation states.');
end

lObservations = log(pObservations + 1e-6);
lTransitions = log(pTransitions + 1e-6);

statePath = 0*pObservations;

lPrev = zeros(nStates, 1);      % Log of previous probabilities
lNew = zeros(nStates,1);        % Log of next probabilities
prevStates = 1:nStates;          % Only need to do this once.
for t=1:nTimes
    for s=1:nStates
        if t==1
            lNew(s) = lObservations(s,t);
        else
            lTotal = lObservations(s,t) + lambda*lTransitions(prevStates,s) + lPrev;
            [bestProb,bestState] = max(lTotal);
            if 0 && t == 424 && s == 42
                fprintf('Got time %d\n', t);
            end
            lNew(s) = bestProb;
            statePath(s,t) = bestState;
        end
    end
    lPrev = lNew;
end

bestPath = zeros(1,nTimes);
[~,i] = max(lNew);
bestPath(nTimes) = i;
for t=nTimes-1:-1:1
    bestPath(t) = statePath(bestPath(t+1),t+1);
end

if interactive> 0 || nargin < 2
    imagesc(pObservations); axis xy; colorbar;
    hold on; plot(1:length(bestPath),bestPath); hold off
    xlabel('Time'); ylabel('State'); title('Viterbi Decoding Test');
    if interactive > 0
        [frameNumber,pitchState] = ginput(1);
        frameNumber = round(frameNumber);
        pitchState = round(pitchState);
        bestPath = nan*zeros(1,nTimes);
        bestPath(frameNumber) = pitchState;
        for t=frameNumber-1:-1:1
            bestPath(t) = statePath(bestPath(t+1),t+1);
        end
    end
    
end;

imagesc(pObservations.^0.5); axis xy; % colorbar;
hold on; plot(1:length(bestPath), bestPath, 'k'), hold off;
ylabel('Pitch Class'); 
title('Viterbi Decoder Output');
    

function [pObservations, pTransitions] = TestViterbi()
nStates = 20;
nTimes = 50;
pObservations = 15*rand(nStates, nTimes);
for t=1:nTimes
    s = (5+3*sin(2*t/nTimes*2*pi));
    h1 = round(s);
    h2 = round(2*s);
    pObservations(h1,t) = pObservations(h1,t)+10.0;
    pObservations(h2,t) = pObservations(h2,t)+5.0;
end
pObservations = (pObservations-min(min(pObservations)))/...
    (max(max(pObservations))-min(min(pObservations)));
pTransitions = eye(nStates);
for i=1:nStates-1
    pTransitions = pTransitions + (1/2)^(i)*diag(ones(nStates-i,1),-i);
    pTransitions = pTransitions + (1/2)^(i)*diag(ones(nStates-i,1),i);
end
    
pTransitions = pTransitions + .1*randn(nStates,nStates);
pTransitions = min(1.0, max(0.0, pTransitions));
